@@ -7,11 +7,12 @@ import matplotlib.pyplot as plt
 import numpy as np
 from mpl_toolkits.mplot3d import Axes3D
 
-from . import base
-from . import utils
-from .. import core
-from .. import errors
-from .. import profile
+from brainpy import backend
+from brainpy import errors
+from brainpy.analysis import base
+from brainpy.analysis import stability
+from brainpy.analysis import utils
+from brainpy.analysis.trajectory import Trajectory
 
 __all__ = [
     'Bifurcation',
@@ -37,18 +38,68 @@ class Bifurcation(object):
     Parameters
     ----------
 
-    model :  NeuType
-        An abstract neuronal type defined in BrainPy.
+    integrals : function, functions
+        The integral functions defined with `brainpy.odeint` or
+        `brainpy.sdeint` or `brainpy.ddeint`, or `brainpy.fdeint`.
+    target_vars : dict
+        The target dynamical variables. It must a dictionary which
+        specifies the boundary of the variables: `{'var1': [min, max]}`.
+    fixed_vars : dict
+        The fixed variables. It must a fixed value with the format of `{'var1': value}`.
+    target_pars : dict, optional
+        The parameters which can be dynamical varied. It must be a dictionary which
+        specifies the boundary of the variables: `{'par1': [min, max]}`
+    pars_update : dict, optional
+        The parameters to update. Or, they can be treated as staitic parameters.
+        Same with the `fixed_vars`, they are must fixed values with the format of
+        `{'par1': value}`.
+    numerical_resolution : float, dict
+        The resolution for numerical iterative solvers. Default is 0.1.
+        It can set the numerical resolution of dynamical variables or dynamical parameters.
+        For example, set ``numerical_resolution=0.1`` will generalize it to all
+        variables and parameters;
+        set ``numerical_resolution={var1: 0.1, var2: 0.2, par1: 0.1, par2: 0.05}`` will
+        specify the particular resolutions to variables and parameters.
+        Moreover, you can also set
+        ``numerical_resolution={var1: np.array([...]), var2: 0.1}`` to specify the
+        search points need to explore for variable `var1`. This will be useful to
+        set sense search points at some inflection points.
+    options : dict, optional
+        The other setting parameters, which includes:
 
+            perturbation
+                float. The small perturbation used to solve the function derivatives.
+            sympy_solver_timeout
+                float, with the unit of second. The maximum time allowed to use sympy solver
+                to get the variable relationship.
+            escape_sympy_solver
+                bool. Whether escape to use sympy solver, and directly use numerical optimization
+                method to solve the nullcline and fixed points.
+            lim_scale
+                float. The axis limit scale factor. Default is 1.05. The setting means
+                the axes will be clipped to ``[var_min * (1-lim_scale)/2, var_max * (var_max-1)/2]``.
+
+        The parameters which are usefull for two-dimensional bifurcation analysis:
+
+            shgo_args
+                dict. Arguments of `shgo` optimization method, which can be used to set the
+                fields of: constraints, n, iters, callback, minimizer_kwargs, options,
+                sampling_method.
+            show_shgo
+                bool. whether print the shgo's value.
+            fl_tol
+                float. The tolerance of the function value to recognize it as a candidate of
+                function root point.
+            xl_tol
+                float. The tolerance of the l2 norm distances between this point and previous
+                points. If the norm distances are all bigger than `xl_tol` means this
+                point belong to a new function root point.
     """
 
-    def __init__(self, model, target_pars, target_vars, fixed_vars=None, pars_update=None,
+    def __init__(self, integrals, target_pars, target_vars, fixed_vars=None, pars_update=None,
                  numerical_resolution=0.1, options=None):
-
         # check "model"
-        if not isinstance(model, core.NeuType):
-            raise errors.ModelUseError('Bifurcation analysis only support neuron type model.')
-        self.model = model
+        self.model = utils.transform_integrals_to_model(integrals)
 
         # check "target_pars"
         if not isinstance(target_pars, dict):
@@ -81,13 +132,13 @@ class Bifurcation(object):
             raise errors.ModelUseError('"pars_update" must be a dict the format of: '
                                        '{"Par A": A_value, "Par B": B_value}')
         for key in pars_update.keys():
-            if key not in model.step_scopes:
-                raise errors.ModelUseError(f'"{key}" is not a valid parameter in "{model.name}" model. ')
+            if (key not in self.model.scopes) and (key not in self.model.parameters):
+                raise errors.ModelUseError(f'"{key}" is not a valid parameter in "{integrals}". ')
         self.pars_update = pars_update
 
         # bifurcation analysis
         if len(self.target_vars) == 1:
-            self.analyzer = _Bifurcation1D(model=model,
+            self.analyzer = _Bifurcation1D(model_or_integrals=self.model,
                                            target_pars=target_pars,
                                            target_vars=target_vars,
                                            fixed_vars=fixed_vars,
@@ -96,7 +147,7 @@ class Bifurcation(object):
                                            options=options)
 
         elif len(self.target_vars) == 2:
-            self.analyzer = _Bifurcation2D(model=model,
+            self.analyzer = _Bifurcation2D(model_or_integrals=self.model,
                                            target_pars=target_pars,
                                            target_vars=target_vars,
                                            fixed_vars=fixed_vars,
@@ -108,18 +159,57 @@ class Bifurcation(object):
             raise errors.ModelUseError(f'Cannot analyze three dimensional system: {self.target_vars}')
 
     def plot_bifurcation(self, *args, **kwargs):
-        self.analyzer.plot_bifurcation(*args, **kwargs)
+        """Plot bifurcation, which support bifurcation analysis of
+        co-dimension 1 and co-dimension 2.
+
+        Parameters
+        ----------
+        show : bool
+            Whether show the bifurcation figure.
+
+        Returns
+        -------
+        points : dict
+            The bifurcation points which specifies their fixed points
+            and corresponding stability.
+        """
+        return self.analyzer.plot_bifurcation(*args, **kwargs)
+
+    def plot_limit_cycle_by_sim(self, var, duration=100, inputs=(), plot_style=None, tol=0.001, show=False):
+        """Plot limit cycles by the simulation results.
+
+        This function help users plot the limit cycles through the simulation results,
+        in which the periodic signals will be automatically found and then treated them
+        as the candidate of limit cycles.
+
+        Parameters
+        ----------
+        var : str
+            The target variable to found its limit cycles.
+        duration : int, float, tuple, list
+            The simulation duration.
+        inputs : tuple, list
+            The simulation inputs.
+        plot_style : dict
+            The limit cycle plotting style settings.
+        tol : float
+            The tolerance to found periodic signals.
+        show : bool
+            Whether show the figure.
+        """
+        self.analyzer.plot_limit_cycle_by_sim(var=var, duration=duration, inputs=inputs,
+                                              plot_style=plot_style, tol=tol, show=show)
 
 
-class _Bifurcation1D(base.Base1DNeuronAnalyzer):
+class _Bifurcation1D(base.Base1DAnalyzer):
     """Bifurcation analysis of 1D system.
 
     Using this class, we can make co-dimension1 or co-dimension2 bifurcation analysis.
     """
 
-    def __init__(self, model, target_pars, target_vars, fixed_vars=None,
+    def __init__(self, model_or_integrals, target_pars, target_vars, fixed_vars=None,
                  pars_update=None, numerical_resolution=0.1, options=None):
-        super(_Bifurcation1D, self).__init__(model=model,
+        super(_Bifurcation1D, self).__init__(model_or_integrals=model_or_integrals,
                                              target_pars=target_pars,
                                              target_vars=target_vars,
                                              fixed_vars=fixed_vars,
@@ -134,7 +224,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
         f_dfdx = self.get_f_dfdx()
 
         if len(self.target_pars) == 1:
-            container = {c: {'p': [], 'x': []} for c in utils.get_1d_classification()}
+            container = {c: {'p': [], 'x': []} for c in stability.get_1d_stability_types()}
 
             # fixed point
             par_a = self.dpar_names[0]
@@ -142,7 +232,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
                 xs = f_fixed_point(p)
                 for x in xs:
                     dfdx = f_dfdx(x, p)
-                    fp_type = utils.stability_analysis(dfdx)
+                    fp_type = stability.stability_analysis(dfdx)
                     container[fp_type]['p'].append(p)
                     container[fp_type]['x'].append(x)
 
@@ -150,7 +240,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
             plt.figure(self.x_var)
             for fp_type, points in container.items():
                 if len(points['x']):
-                    plot_style = utils.plot_scheme[fp_type]
+                    plot_style = stability.plot_scheme[fp_type]
                     plt.plot(points['p'], points['x'], '.', **plot_style, label=fp_type)
             plt.xlabel(par_a)
             plt.ylabel(self.x_var)
@@ -164,7 +254,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
                 plt.show()
 
         elif len(self.target_pars) == 2:
-            container = {c: {'p0': [], 'p1': [], 'x': []} for c in utils.get_1d_classification()}
+            container = {c: {'p0': [], 'p1': [], 'x': []} for c in stability.get_1d_stability_types()}
 
             # fixed point
             for p0 in self.resolutions[self.dpar_names[0]]:
@@ -172,7 +262,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
                     xs = f_fixed_point(p0, p1)
                     for x in xs:
                         dfdx = f_dfdx(x, p0, p1)
-                        fp_type = utils.stability_analysis(dfdx)
+                        fp_type = stability.stability_analysis(dfdx)
                         container[fp_type]['p0'].append(p0)
                         container[fp_type]['p1'].append(p1)
                         container[fp_type]['x'].append(x)
@@ -182,7 +272,7 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
             ax = fig.gca(projection='3d')
             for fp_type, points in container.items():
                 if len(points['x']):
-                    plot_style = utils.plot_scheme[fp_type]
+                    plot_style = stability.plot_scheme[fp_type]
                     xs = points['p0']
                     ys = points['p1']
                     zs = points['x']
@@ -211,16 +301,15 @@ class _Bifurcation1D(base.Base1DNeuronAnalyzer):
         raise NotImplementedError('1D phase plane do not support plot_limit_cycle_by_sim.')
 
 
-
-class _Bifurcation2D(base.Base2DNeuronAnalyzer):
+class _Bifurcation2D(base.Base2DAnalyzer):
     """Bifurcation analysis of 2D system.
 
     Using this class, we can make co-dimension1 or co-dimension2 bifurcation analysis.
     """
 
-    def __init__(self, model, target_pars, target_vars, fixed_vars=None,
+    def __init__(self, model_or_integrals, target_pars, target_vars, fixed_vars=None,
                  pars_update=None, numerical_resolution=0.1, options=None):
-        super(_Bifurcation2D, self).__init__(model=model,
+        super(_Bifurcation2D, self).__init__(model_or_integrals=model_or_integrals,
                                              target_pars=target_pars,
                                              target_vars=target_vars,
                                              fixed_vars=fixed_vars,
@@ -240,14 +329,14 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
         # bifurcation analysis of co-dimension 1
         if len(self.target_pars) == 1:
             container = {c: {'p': [], self.x_var: [], self.y_var: []}
-                         for c in utils.get_2d_classification()}
+                         for c in stability.get_2d_stability_types()}
 
             # fixed point
             for p in self.resolutions[self.dpar_names[0]]:
                 xs, ys = f_fixed_point(p)
                 for x, y in zip(xs, ys):
                     dfdx = f_jacobian(x, y, p)
-                    fp_type = utils.stability_analysis(dfdx)
+                    fp_type = stability.stability_analysis(dfdx)
                     container[fp_type]['p'].append(p)
                     container[fp_type][self.x_var].append(x)
                     container[fp_type][self.y_var].append(y)
@@ -257,7 +346,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
                 plt.figure(var)
                 for fp_type, points in container.items():
                     if len(points['p']):
-                        plot_style = utils.plot_scheme[fp_type]
+                        plot_style = stability.plot_scheme[fp_type]
                         plt.plot(points['p'], points[var], '.', **plot_style, label=fp_type)
                 plt.xlabel(self.dpar_names[0])
                 plt.ylabel(var)
@@ -273,7 +362,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
         # bifurcation analysis of co-dimension 2
         elif len(self.target_pars) == 2:
             container = {c: {'p0': [], 'p1': [], self.x_var: [], self.y_var: []}
-                         for c in utils.get_2d_classification()}
+                         for c in stability.get_2d_stability_types()}
 
             # fixed point
             for p0 in self.resolutions[self.dpar_names[0]]:
@@ -281,7 +370,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
                     xs, ys = f_fixed_point(p0, p1)
                     for x, y in zip(xs, ys):
                         dfdx = f_jacobian(x, y, p0, p1)
-                        fp_type = utils.stability_analysis(dfdx)
+                        fp_type = stability.stability_analysis(dfdx)
                         container[fp_type]['p0'].append(p0)
                         container[fp_type]['p1'].append(p1)
                         container[fp_type][self.x_var].append(x)
@@ -293,7 +382,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
                 ax = fig.gca(projection='3d')
                 for fp_type, points in container.items():
                     if len(points['p0']):
-                        plot_style = utils.plot_scheme[fp_type]
+                        plot_style = stability.plot_scheme[fp_type]
                         xs = points['p0']
                         ys = points['p1']
                         zs = points[var]
@@ -318,7 +407,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
 
         self.fixed_points = container
         return container
-    
+
     def plot_limit_cycle_by_sim(self, var, duration=100, inputs=(), plot_style=None, tol=0.001, show=False):
         print('plot limit cycle ...')
 
@@ -334,7 +423,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
         all_xs, all_ys, all_p0, all_p1 = [], [], [], []
 
         # unstable node
-        unstable_node = self.fixed_points[utils._2D_UNSTABLE_NODE]
+        unstable_node = self.fixed_points[stability.UNSTABLE_NODE_2D]
         all_xs.extend(unstable_node[self.x_var])
         all_ys.extend(unstable_node[self.y_var])
         if len(self.dpar_names) == 1:
@@ -346,7 +435,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
             raise ValueError
 
         # unstable focus
-        unstable_focus = self.fixed_points[utils._2D_UNSTABLE_FOCUS]
+        unstable_focus = self.fixed_points[stability.UNSTABLE_FOCUS_2D]
         all_xs.extend(unstable_focus[self.x_var])
         all_ys.extend(unstable_focus[self.y_var])
         if len(self.dpar_names) == 1:
@@ -373,23 +462,13 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
 
         # initialize neuron group
         length = all_xs.shape[0]
-        group = core.NeuGroup(self.model,
-                              geometry=length,
-                              monitors=self.dvar_names,
-                              pars_update=self.pars_update)
-
-        # group initial state
-        group.ST[self.x_var] = all_xs
-        group.ST[self.y_var] = all_ys
-        for key, val in fixed_vars.items():
-            if key in group.ST:
-                group.ST[key] = val
-
-        # run neuron group
-        group.runner = core.TrajectoryRunner(group,
-                                             target_vars=self.dvar_names,
-                                             fixed_vars=fixed_vars)
-        group.run(duration=duration, inputs=inputs)
+        traj_group = Trajectory(size=length,
+                                integrals=self.model.integrals,
+                                target_vars={self.x_var: all_xs, self.y_var: all_ys},
+                                fixed_vars=fixed_vars,
+                                pars_update=self.pars_update,
+                                scope=self.model.scopes)
+        traj_group.run(duration=duration)
 
         # find limit cycles
         limit_cycle_max = []
@@ -398,7 +477,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
         p0_limit_cycle = []
         p1_limit_cycle = []
         for i in range(length):
-            data = group.mon[var][:, i]
+            data = traj_group.mon[var][:, i]
             max_index = utils.find_indexes_of_limit_cycle_max(data, tol=tol)
             if max_index[0] != -1:
                 x_cycle = data[max_index[0]: max_index[1]]
@@ -434,9 +513,7 @@ class _Bifurcation2D(base.Base2DNeuronAnalyzer):
         if show:
             plt.show()
 
-        del group.ST
-        del group.mon
-        del group
+        del traj_group
         gc.collect()
 
 
@@ -448,6 +525,50 @@ class FastSlowBifurcation(object):
     and then study how the different value of slow variables affect the
     bifurcation of the fast sub-system.
 
+    Parameters
+    ----------
+
+    integrals : function, functions
+        The integral functions defined with `brainpy.odeint` or
+        `brainpy.sdeint` or `brainpy.ddeint`, or `brainpy.fdeint`.
+    fast_vars : dict
+        The fast dynamical variables. It must a dictionary which
+        specifies the boundary of the variables: `{'var1': [min, max]}`.
+    slow_vars : dict
+        The slow dynamical variables. It must a dictionary which
+        specifies the boundary of the variables: `{'var1': [min, max]}`.
+    fixed_vars : dict
+        The fixed variables. It must a fixed value with the format of `{'var1': value}`.
+    pars_update : dict, optional
+        The parameters to update. Or, they can be treated as staitic parameters.
+        Same with the `fixed_vars`, they are must fixed values with the format of
+        `{'par1': value}`.
+    numerical_resolution : float, dict
+        The resolution for numerical iterative solvers. Default is 0.1.
+        It can set the numerical resolution of dynamical variables or dynamical parameters.
+        For example, set ``numerical_resolution=0.1`` will generalize it to all
+        variables and parameters;
+        set ``numerical_resolution={var1: 0.1, var2: 0.2, par1: 0.1, par2: 0.05}`` will
+        specify the particular resolutions to variables and parameters.
+        Moreover, you can also set
+        ``numerical_resolution={var1: np.array([...]), var2: 0.1}`` to specify the
+        search points need to explore for variable `var1`. This will be useful to
+        set sense search points at some inflection points.
+    options : dict, optional
+        The other setting parameters, which includes:
+
+            perturbation
+                float. The small perturbation used to solve the function derivatives.
+            sympy_solver_timeout
+                float, with the unit of second. The maximum time allowed to use sympy solver
+                to get the variable relationship.
+            escape_sympy_solver
+                bool. Whether escape to use sympy solver, and directly use numerical optimization
+                method to solve the nullcline and fixed points.
+            lim_scale
+                float. The axis limit scale factor. Default is 1.05. The setting means
+                the axes will be clipped to ``[var_min * (1-lim_scale)/2, var_max * (var_max-1)/2]``.
+
     References
     ----------
 
@@ -464,12 +585,10 @@ class FastSlowBifurcation(object):
 
     """
 
-    def __init__(self, model, fast_vars, slow_vars, fixed_vars=None,
+    def __init__(self, integrals, fast_vars, slow_vars, fixed_vars=None,
                  pars_update=None, numerical_resolution=0.1, options=None):
         # check "model"
-        if not isinstance(model, core.NeuType):
-            raise errors.ModelUseError('FastSlowBifurcation only support neuron type model.')
-        self.model = model
+        self.model = utils.transform_integrals_to_model(integrals)
 
         # check "fast_vars"
         if not isinstance(fast_vars, dict):
@@ -490,6 +609,9 @@ class FastSlowBifurcation(object):
         if len(slow_vars) > 2:
             raise errors.ModelUseError("FastSlowBifurcation can only analyze the system with less "
                                        "than two-variable slow subsystem.")
+        for key in self.slow_vars:
+            self.model.variables.remove(key)
+            self.model.parameters.append(key)
 
         # check "fixed_vars"
         if fixed_vars is None:
@@ -506,13 +628,13 @@ class FastSlowBifurcation(object):
             raise errors.ModelUseError('"pars_update" must be a dict the format of: '
                                        '{"Par A": A_value, "Par B": B_value}')
         for key in pars_update.keys():
-            if key not in model.step_scopes:
-                raise errors.ModelUseError(f'"{key}" is not a valid parameter in "{model.name}" model. ')
+            if (key not in self.model.scopes) and (key not in self.model.parameters):
+                raise errors.ModelUseError(f'"{key}" is not a valid parameter in "{integrals}" model. ')
         self.pars_update = pars_update
 
         # bifurcation analysis
         if len(self.fast_vars) == 1:
-            self.analyzer = _FastSlow1D(model=model,
+            self.analyzer = _FastSlow1D(model_or_integrals=self.model,
                                         fast_vars=fast_vars,
                                         slow_vars=slow_vars,
                                         fixed_vars=fixed_vars,
@@ -521,7 +643,7 @@ class FastSlowBifurcation(object):
                                         options=options)
 
         elif len(self.fast_vars) == 2:
-            self.analyzer = _FastSlow2D(model=model,
+            self.analyzer = _FastSlow2D(model_or_integrals=self.model,
                                         fast_vars=fast_vars,
                                         slow_vars=slow_vars,
                                         fixed_vars=fixed_vars,
@@ -533,19 +655,83 @@ class FastSlowBifurcation(object):
             raise errors.ModelUseError(f'Cannot analyze {len(fast_vars)} dimensional fast system.')
 
     def plot_bifurcation(self, *args, **kwargs):
-        self.analyzer.plot_bifurcation(*args, **kwargs)
+        """Plot bifurcation.
+
+        Parameters
+        ----------
+        show : bool
+            Whether show the bifurcation figure.
+
+        Returns
+        -------
+        points : dict
+            The bifurcation points which specifies their fixed points
+            and corresponding stability.
+        """
+        return self.analyzer.plot_bifurcation(*args, **kwargs)
 
     def plot_trajectory(self, *args, **kwargs):
+        """Plot trajectory.
+
+        This function helps users to plot specific trajectories.
+
+        Parameters
+        ----------
+        initials : list, tuple
+            The initial value setting of the targets. It can be a tuple/list of floats to specify
+            each value of dynamical variables (for example, ``(a, b)``). It can also be a
+            tuple/list of tuple to specify multiple initial values (for example,
+            ``[(a1, b1), (a2, b2)]``).
+        duration : int, float, tuple, list
+            The running duration. Same with the ``duration`` in ``NeuGroup.run()``.
+            It can be a int/float (``t_end``) to specify the same running end time,
+            or it can be a tuple/list of int/float (``(t_start, t_end)``) to specify
+            the start and end simulation time. Or, it can be a list of tuple
+            (``[(t1_start, t1_end), (t2_start, t2_end)]``) to specify the specific
+            start and end simulation time for each initial value.
+        plot_duration : tuple/list of tuple, optional
+            The duration to plot. It can be a tuple with ``(start, end)``. It can
+            also be a list of tuple ``[(start1, end1), (start2, end2)]`` to specify
+            the plot duration for each initial value running.
+        show : bool
+            Whether show or not.
+        """
         self.analyzer.plot_trajectory(*args, **kwargs)
 
     def plot_limit_cycle_by_sim(self, *args, **kwargs):
+        """Plot limit cycles by the simulation results.
+
+        This function help users plot the limit cycles through the simulation results,
+        in which the periodic signals will be automatically found and then treated them
+        as the candidate of limit cycles.
+
+        Parameters
+        ----------
+        var : str
+            The target variable to found its limit cycles.
+        duration : int, float, tuple, list
+            The simulation duration.
+        inputs : tuple, list
+            The simulation inputs.
+        plot_style : dict
+            The limit cycle plotting style settings.
+        tol : float
+            The tolerance to found periodic signals.
+        show : bool
+            Whether show the figure.
+        """
         self.analyzer.plot_limit_cycle_by_sim(*args, **kwargs)
 
 
 class _FastSlowTrajectory(object):
-    def __init__(self, model, fast_vars, slow_vars, fixed_vars=None,
+    def __init__(self, model_or_intgs, fast_vars, slow_vars, fixed_vars=None,
                  pars_update=None, **kwargs):
-        self.model = model
+        if isinstance(model_or_intgs, utils.DynamicModel):
+            self.model = model_or_intgs
+        elif (isinstance(model_or_intgs, (list, tuple)) and callable(model_or_intgs[0])) or callable(model_or_intgs):
+            self.model = utils.transform_integrals_to_model(model_or_intgs)
+        else:
+            raise ValueError
         self.fast_vars = fast_vars
         self.slow_vars = slow_vars
         self.fixed_vars = fixed_vars
@@ -567,20 +753,7 @@ class _FastSlowTrajectory(object):
         else:
             self.slow_var_names = list(sorted(slow_vars.keys()))
 
-        # cannot update dynamical parameters
-        all_vars = self.fast_var_names + self.slow_var_names
-        self.traj_group = core.NeuGroup(model,
-                                        geometry=1,
-                                        monitors=all_vars,
-                                        pars_update=pars_update)
-        self.traj_group.runner = core.TrajectoryRunner(self.traj_group,
-                                                       target_vars=all_vars,
-                                                       fixed_vars=fixed_vars)
-        self.traj_initial = {key: val[0] for key, val in self.traj_group.ST.items()
-                             if not key.startswith('_')}
-        self.traj_net = core.Network(self.traj_group)
-
-    def plot_trajectory(self, initials, duration, plot_duration=None, inputs=(), show=False):
+    def plot_trajectory(self, initials, duration, plot_duration=None, show=False):
         """Plot trajectories according to the settings.
 
         Parameters
@@ -601,8 +774,6 @@ class _FastSlowTrajectory(object):
             The duration to plot. It can be a tuple with ``(start, end)``. It can
             also be a list of tuple ``[(start1, end1), (start2, end2)]`` to specify
             the plot duration for each initial value running.
-        inputs : tuple, list
-            The inputs to the model. Same with the ``inputs`` in ``NeuGroup.run()``
         show : bool
             Whether show or not.
         """
@@ -645,29 +816,15 @@ class _FastSlowTrajectory(object):
         else:
             assert len(plot_duration) == len(initials)
 
-        # 4. format the inputs
-        if len(inputs):
-            if isinstance(inputs[0], (tuple, list)):
-                inputs = [(self.traj_group,) + tuple(input) for input in inputs]
-            elif isinstance(inputs[0], str):
-                inputs = [(self.traj_group,) + tuple(inputs)]
-            else:
-                raise errors.ModelUseError()
-
         # 5. run the network
         for init_i, initial in enumerate(initials):
-            #   5.1 set the initial value
-            for key, val in self.traj_initial.items():
-                self.traj_group.ST[key] = val
-            for key in all_vars:
-                self.traj_group.ST[key] = initial[key]
-            for key, val in self.fixed_vars.items():
-                if key in self.traj_group.ST:
-                    self.traj_group.ST[key] = val
-
-            #   5.2 run the model
-            self.traj_net.run(duration=duration[init_i], inputs=inputs,
-                              report=False, data_to_host=True, verbose=False)
+            traj_group = Trajectory(size=1,
+                                    integrals=self.model.integrals,
+                                    target_vars=initial,
+                                    fixed_vars=self.fixed_vars,
+                                    pars_update=self.pars_update,
+                                    scope=self.model.scopes)
+            traj_group.run(duration=duration[init_i], report=False)
 
             #   5.3 legend
             legend = f'$traj_{init_i}$: '
@@ -676,13 +833,13 @@ class _FastSlowTrajectory(object):
             legend = legend[:-2]
 
             #   5.4 trajectory
-            start = int(plot_duration[init_i][0] / profile.get_dt())
-            end = int(plot_duration[init_i][1] / profile.get_dt())
+            start = int(plot_duration[init_i][0] / backend.get_dt())
+            end = int(plot_duration[init_i][1] / backend.get_dt())
 
             #   5.5 visualization
             for var_name in self.fast_var_names:
-                s0 = self.traj_group.mon[self.slow_var_names[0]][start: end, 0]
-                fast = self.traj_group.mon[var_name][start: end, 0]
+                s0 = traj_group.mon[self.slow_var_names[0]][start: end, 0]
+                fast = traj_group.mon[var_name][start: end, 0]
 
                 fig = plt.figure(var_name)
                 if len(self.slow_var_names) == 1:
@@ -695,7 +852,7 @@ class _FastSlowTrajectory(object):
 
                 elif len(self.slow_var_names) == 2:
                     fig.gca(projection='3d')
-                    s1 = self.traj_group.mon[self.slow_var_names[1]][start: end, 0]
+                    s1 = traj_group.mon[self.slow_var_names[1]][start: end, 0]
                     plt.plot(s0, s1, fast, label=legend)
                 else:
                     raise errors.AnalyzerError
@@ -725,18 +882,17 @@ class _FastSlowTrajectory(object):
             plt.show()
 
 
-
 class _FastSlow1D(_Bifurcation1D):
-    def __init__(self, model, fast_vars, slow_vars, fixed_vars=None,
+    def __init__(self, model_or_integrals, fast_vars, slow_vars, fixed_vars=None,
                  pars_update=None, numerical_resolution=0.1, options=None):
-        super(_FastSlow1D, self).__init__(model=model,
+        super(_FastSlow1D, self).__init__(model_or_integrals=model_or_integrals,
                                           target_pars=slow_vars,
                                           target_vars=fast_vars,
                                           fixed_vars=fixed_vars,
                                           pars_update=pars_update,
                                           numerical_resolution=numerical_resolution,
                                           options=options)
-        self.traj = _FastSlowTrajectory(model=model,
+        self.traj = _FastSlowTrajectory(model_or_intgs=model_or_integrals,
                                         fast_vars=fast_vars,
                                         slow_vars=slow_vars,
                                         fixed_vars=fixed_vars,
@@ -748,23 +904,23 @@ class _FastSlow1D(_Bifurcation1D):
         self.traj.plot_trajectory(*args, **kwargs)
 
     def plot_bifurcation(self, *args, **kwargs):
-        super(_FastSlow1D, self).plot_bifurcation(*args, **kwargs)
+        return super(_FastSlow1D, self).plot_bifurcation(*args, **kwargs)
 
     def plot_limit_cycle_by_sim(self, *args, **kwargs):
         super(_FastSlow1D, self).plot_limit_cycle_by_sim(*args, **kwargs)
 
 
 class _FastSlow2D(_Bifurcation2D):
-    def __init__(self, model, fast_vars, slow_vars, fixed_vars=None,
+    def __init__(self, model_or_integrals, fast_vars, slow_vars, fixed_vars=None,
                  pars_update=None, numerical_resolution=0.1, options=None):
-        super(_FastSlow2D, self).__init__(model=model,
+        super(_FastSlow2D, self).__init__(model_or_integrals=model_or_integrals,
                                           target_pars=slow_vars,
                                           target_vars=fast_vars,
                                           fixed_vars=fixed_vars,
                                           pars_update=pars_update,
                                           numerical_resolution=numerical_resolution,
                                           options=options)
-        self.traj = _FastSlowTrajectory(model=model,
+        self.traj = _FastSlowTrajectory(model_or_intgs=model_or_integrals,
                                         fast_vars=fast_vars,
                                         slow_vars=slow_vars,
                                         fixed_vars=fixed_vars,
@@ -776,7 +932,7 @@ class _FastSlow2D(_Bifurcation2D):
         self.traj.plot_trajectory(*args, **kwargs)
 
     def plot_bifurcation(self, *args, **kwargs):
-        super(_FastSlow2D, self).plot_bifurcation(*args, **kwargs)
+        return super(_FastSlow2D, self).plot_bifurcation(*args, **kwargs)
 
     def plot_limit_cycle_by_sim(self, *args, **kwargs):
         super(_FastSlow2D, self).plot_limit_cycle_by_sim(*args, **kwargs)
