@@ -21,20 +21,20 @@ __all__ = [
 ]
 
 
-def _base(A, B, C, f, show_code, dt, var_type, im_return):
+def _base(A, B, C, f, show_code, dt, var_type):
     dt = backend.get_dt() if dt is None else dt
     show_code = False if show_code is None else show_code
     var_type = constants.SCALAR_VAR if var_type is None else var_type
 
     if f is None:
         return lambda f: general_rk_wrapper(f=f, show_code=show_code, dt=dt, A=A, B=B, C=C,
-                                            var_type=var_type, im_return=im_return)
+                                            var_type=var_type)
     else:
         return general_rk_wrapper(f=f, show_code=show_code, dt=dt, A=A, B=B, C=C,
-                                  var_type=var_type, im_return=im_return)
+                                  var_type=var_type)
 
 
-def euler(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def euler(f=None, show_code=None, dt=None, var_type=None):
     """The Euler method is first order. The lack of stability
         and accuracy limits its popularity mainly to use as a
         simple introductory example of a numeric solution method.
@@ -42,11 +42,16 @@ def euler(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), ]
     B = [1]
     C = [0]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def midpoint(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def midpoint(f=None, show_code=None, dt=None, var_type=None):
     """midpoint method for ordinary differential equations.
 
     The (explicit) midpoint method is a second-order method
@@ -70,11 +75,16 @@ def midpoint(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (0.5,)]
     B = [0, 1]
     C = [0, 0.5]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def heun2(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def heun2(f=None, show_code=None, dt=None, var_type=None):
     """Heun's method for ordinary differential equations.
 
     Heun's method is a second-order method with two stages.
@@ -99,11 +109,16 @@ def heun2(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (1,)]
     B = [0.5, 0.5]
     C = [0, 1]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def ralston2(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def ralston2(f=None, show_code=None, dt=None, var_type=None):
     """Ralston's method for ordinary differential equations.
 
     Ralston's method is a second-order method with two stages and
@@ -126,11 +141,16 @@ def ralston2(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), ('2/3',)]
     B = [0.25, 0.75]
     C = [0, '2/3']
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def rk2(f=None, show_code=None, dt=None, beta=None, var_type=None, im_return=()):
+def rk2(f=None, show_code=None, dt=None, beta=None, var_type=None):
     """Runge–Kutta methods for ordinary differential equations.
 
     Generic second-order method.
@@ -156,13 +176,13 @@ def rk2(f=None, show_code=None, dt=None, beta=None, var_type=None, im_return=())
 
     if f is None:
         return lambda f: rk2_wrapper(f, show_code=show_code, dt=dt, beta=beta,
-                                     var_type=var_type, im_return=im_return)
+                                     var_type=var_type)
     else:
         return rk2_wrapper(f, show_code=show_code, dt=dt, beta=beta,
-                           var_type=var_type, im_return=im_return)
+                           var_type=var_type)
 
 
-def rk3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def rk3(f=None, show_code=None, dt=None, var_type=None):
     """Classical third-order Runge-Kutta method for ordinary differential equations.
 
     It has the characteristics of:
@@ -184,11 +204,16 @@ def rk3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (0.5,), (-1, 2)]
     B = ['1/6', '2/3', '1/6']
     C = [0, 0.5, 1]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def heun3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def heun3(f=None, show_code=None, dt=None, var_type=None):
     """Heun's third-order method for ordinary differential equations.
 
     It has the characteristics of:
@@ -210,11 +235,16 @@ def heun3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), ('1/3',), (0, '2/3')]
     B = [0.25, 0, 0.75]
     C = [0, '1/3', '2/3']
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def ralston3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def ralston3(f=None, show_code=None, dt=None, var_type=None):
     """Ralston's third-order method for ordinary differential equations.
 
     It has the characteristics of:
@@ -242,11 +272,16 @@ def ralston3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (0.5,), (0, 0.75)]
     B = ['2/9', '1/3', '4/9']
     C = [0, 0.5, 0.75]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def ssprk3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def ssprk3(f=None, show_code=None, dt=None, var_type=None):
     """Third-order Strong Stability Preserving Runge-Kutta (SSPRK3).
 
     It has the characteristics of:
@@ -268,11 +303,16 @@ def ssprk3(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (1,), (0.25, 0.25)]
     B = ['1/6', '1/6', '2/3']
     C = [0, 1, 0.5]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def rk4(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def rk4(f=None, show_code=None, dt=None, var_type=None):
     """Classical fourth-order Runge-Kutta method for ordinary differential equations.
 
     It has the characteristics of:
@@ -296,11 +336,16 @@ def rk4(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (0.5,), (0., 0.5), (0., 0., 1)]
     B = ['1/6', '1/3', '1/3', '1/6']
     C = [0, 0.5, 0.5, 1]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def ralston4(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def ralston4(f=None, show_code=None, dt=None, var_type=None):
     """Ralston's fourth-order method for ordinary differential equations.
 
     It has the characteristics of:
@@ -329,11 +374,16 @@ def ralston4(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), (.4,), (.29697761, .15875964), (.21810040, -3.05096516, 3.83286476)]
     B = [.17476028, -.55148066, 1.20553560, .17118478]
     C = [0, .4, .45573725, 1]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
 
 
-def rk4_38rule(f=None, show_code=None, dt=None, var_type=None, im_return=()):
+def rk4_38rule(f=None, show_code=None, dt=None, var_type=None):
     """3/8-rule fourth-order method for ordinary differential equations.
 
     This method doesn't have as much notoriety as the "classical" method,
@@ -360,5 +410,10 @@ def rk4_38rule(f=None, show_code=None, dt=None, var_type=None, im_return=()):
     A = [(), ('1/3',), ('-1/3', '1'), (1, -1, 1)]
     B = ['1/8', '3/8', '3/8', '1/8']
     C = [0, '1/3', '2/3', 1]
-    return _base(A=A, B=B, C=C, f=f, show_code=show_code, dt=dt,
-                 var_type=var_type, im_return=im_return)
+    return _base(A=A,
+                 B=B,
+                 C=C,
+                 f=f,
+                 show_code=show_code,
+                 dt=dt,
+                 var_type=var_type)
