@@ -3,4 +3,4 @@
 from .ast2code import *
 from .codes import *
 from .dicts import *
-from .functions import *
+from .decorators import *
