@@ -9,18 +9,18 @@ __all__ = [
 ]
 
 
-def get_figure(n_row, n_col, len_row=3, len_col=6):
+def get_figure(row_num, col_num, row_len=3, col_len=6):
     """Get the constrained_layout figure.
 
     Parameters
     ----------
-    n_row : int
+    row_num : int
         The row number of the figure.
-    n_col : int
+    col_num : int
         The column number of the figure.
-    len_row : int, float
+    row_len : int, float
         The length of each row.
-    len_col : int, float
+    col_len : int, float
         The length of each column.
 
     Returns
@@ -28,6 +28,6 @@ def get_figure(n_row, n_col, len_row=3, len_col=6):
     fig_and_gs : tuple
         Figure and GridSpec.
     """
-    fig = plt.figure(figsize=(n_col * len_col, n_row * len_row), constrained_layout=True)
-    gs = GridSpec(n_row, n_col, figure=fig)
+    fig = plt.figure(figsize=(col_num * col_len, row_num * row_len), constrained_layout=True)
+    gs = GridSpec(row_num, col_num, figure=fig)
     return fig, gs
